#ifndef MATRIX_H
#define MATRIX_H

#define MATRIX_DONE  0
#define MATRIX_UNSET 1
#define MATRIX_NOMEM 2
#define MATRIX_NOVAL 3
#define MATRIX_USERR 4

struct _matrix {
	double *val;
	unsigned short rows;
	unsigned short cols;
};

typedef struct _matrix matrix;

/* function constructss a new matrix */
matrix *matrix_new();

/* funtion destructs an existing matrix */
int matrix_free(matrix **mx);

/* function sets 0 into each matrix element if *val is defined */
int matrix_clear(matrix *mx);

/* function changes the size of matrix */
int matrix_resize(matrix *mx, unsigned short rows, unsigned short cols);

/* function sets val as matrix value, if possible */
int matrix_set(matrix *mx, const double *val, unsigned short rows, unsigned short cols);

/* funcion puts value of element(col, row) from matrix *mx into *dst */
int matrix_get_element(double *dst, const matrix *mx, unsigned short row, unsigned short col);

/* function sets value val of element(col,row) of matrix *mx*/
int matrix_set_element(double val, matrix *mx, unsigned short row, unsigned short col);

/* function copies matrix *src into matrix *dst */
int matrix_copy(matrix *dst, const matrix *src);

/* function reverses sign of each matrix *mx element*/
int matrix_rsign(matrix *mx);

/* function sets *res matrix as result of matrixes *a and *b adding */
int matrix_add_matrix(matrix *res, const matrix *a, const matrix *b);

/* function sets *res matrix as *a subtract *b */
int matrix_sub_matrix(matrix *res, const matrix *a, const matrix *b);

/* function sets *res matrix as result of multiplication of *a by *b */
int matrix_mult_matrix(matrix *res, const matrix *a, const matrix *b);

/* function increases each element of *mx by num */
int matrix_add_number(matrix *mx, double num);

/* function multiplies each element of *mx by num */
int matrix_mult_number(matrix *mx, double num);

/* function transponses *mx matrix */
int matrix_transpose(matrix *mx);
#endif
